<?php 
  
    /*
    Plugin Name: WordPress Persistent Login
    Plugin URI: 
    Description: Keep users logged into your website securely.
    Author: Luke Seager
    Version: 1.1.4
    Author URI: https://lukeseager.com
    */
    
    
    /*  
    Copyright 2018 Luke Seager  (email : info@lukeseager.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
	*/
	
	
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

if ( ! function_exists( 'persistent_login' ) ) :
	
	/* ------------------------------------------------------------------------ *
	 * Load freemius
	 * ------------------------------------------------------------------------ */
			
			include_once( plugin_dir_path(__FILE__) . 'freemius.php' );
	
	
	
	/* ------------------------------------------------------------------------ *
	 * Setup - include plugin and get global vars
	 * ------------------------------------------------------------------------ */
	    
	    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
	        	
	  	// plugin variables
	  	global $tableRef;
	  	$tableRef = 'persistent_logins'; 
	  	
	    // install plugin
	    global $persistent_login_db_version;
			$persistent_login_db_version = '1.1.3';
		
		
		
	/* ------------------------------------------------------------------------ *
	 * Install persistent login
	 * ------------------------------------------------------------------------ */
	 		 
			function persistent_login_install() {
								
				// load required global vars
					global $wpdb;
					global $tableRef;
				
				
				// set table name
					$table = $wpdb->prefix . $tableRef;
						
						
				// create the table
					$charset_collate = '';
					
					if ( ! empty( $wpdb->charset ) ) {
					  $charset_collate = "DEFAULT CHARACTER SET {$wpdb->charset}";
					}
				
					if ( ! empty( $wpdb->collate ) ) {
					  $charset_collate .= " COLLATE {$wpdb->collate}";
					}
				
					$sql = "CREATE TABLE $table (
						id mediumint(9) NOT NULL AUTO_INCREMENT,
						user INT NOT NULL,
						login_key CHAR(40) NOT NULL,
						ip INT(11) NOT NULL,
						user_agent varchar(255) NOT NULL,
						UNIQUE KEY id (id)
					) $charset_collate;";
				
					require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
					dbDelta( $sql );

				
				// add db version for future reference
					global $persistent_login_db_version;
					update_option( 'persistent_login_db_version', $persistent_login_db_version);
				
				
				// set detaults for permissions
					global $wp_roles;
					$options = array();
					foreach( $wp_roles->roles as $key=>$value ) :
						array_push($options, $key);
					endforeach;
					update_option('persistent_login_options_user_access', $options);
					
					
				// send user thank you message
					set_transient( 'persistent_login_install_success', true, 5 );
							
			}
			register_activation_hook( __FILE__, 'persistent_login_install' );

		
	/* ------------------------------------------------------------------------ *
	 * END install persistent login
	 * ------------------------------------------------------------------------ */
	
	
	
	/* ------------------------------------------------------------------------ *
	 * Update DB 1.1.3 persistent login
	 * ------------------------------------------------------------------------ */
	
		// check db version
			function persistent_login_update_db_check() {
			  global $persistent_login_db_version;
			  if (get_option( 'persistent_login_db_version' ) !== $persistent_login_db_version) {
			      persistent_login_update_db();
			  }
			}
			add_action( 'plugins_loaded', 'persistent_login_update_db_check' );
		
		// run update 
			function persistent_login_update_db() {
				
				// load required global vars
					global $wpdb;
					global $tableRef;
				
				// set table name
					$table = $wpdb->prefix . $tableRef;
				
				// fetch charset for db	
					$charset_collate = $wpdb->get_charset_collate();
								
				// run query					
					$table_update = $wpdb->query( 
						"
							ALTER TABLE $table 
							ADD `ip` INT(11) NOT NULL AFTER `login_key`,
							ADD `user_agent` varchar(255) NOT NULL AFTER `ip`
						"
					);
						
				// update db version option
					global $persistent_login_db_version;
					update_option( 'persistent_login_db_version', $persistent_login_db_version);
					
				// send user to update page
					wp_redirect( admin_url( '/options-general.php?page=wp-persistent-login&view=update' ), 301 );
				
			}
	
	/* ------------------------------------------------------------------------ *
	 * END Update DB 1.1.3 persistent login
	 * ------------------------------------------------------------------------ */
	 
	
	
	/* ------------------------------------------------------------------------ *
	 * Install messages
	 * ------------------------------------------------------------------------ */ 
	 
	 		add_action( 'admin_notices', 'persistent_login_notices' );

			function persistent_login_notices(){
			
			  // success
			    if( get_transient( 'persistent_login_install_success' ) ) :
						
						$message = __('Thank you for using Persistent Login!');
						$class = 'notice updated';
						printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) ); 
			      delete_transient( 'persistent_login_install_success' );
			      
			    endif;
			  
			  
			  // failure
			  	if( get_transient( 'persistent_login_install_failed' ) ) :
			  	 
				  	$message = __('There is already a Persistent Login table. The plugin should work, but if you have an issue please re-install the plugin.');
						$class = 'notice notice-warning';
						printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
				    delete_transient( 'persistent_login_install_failed' );
			     
			    endif;
			  
			}
		
	/* ------------------------------------------------------------------------ *
	 * END install messages
	 * ------------------------------------------------------------------------ */
		
		
		
	/* ------------------------------------------------------------------------ *
	 * Uninstall persistent login
	 * ------------------------------------------------------------------------ */
	 
		// setup uninstall cleanup
			function persistent_login_uninstall_cleanup() {
				
				// remove database options
					$options = array(
						'persistent_login_db_version',
						'persistent_login_options_user_access'
					);
					foreach( $options as $option ) :  
						delete_option($option);
					endforeach;
				
				// remove the table from the db
					global $wpdb;
			    $table = $wpdb->prefix . 'persistent_logins';
			    $wpdb->query("DROP TABLE IF EXISTS $table");
			    
			}
			persistent_login()->add_action('after_uninstall', 'persistent_login_uninstall_cleanup');
			
	/* ------------------------------------------------------------------------ *
	 * END uninstall persistent login
	 * ------------------------------------------------------------------------ */
		
		
		
	/* ------------------------------------------------------------------------ *
	 * Plugin admin
	 * ------------------------------------------------------------------------ */
	 
		// add settings button to plugins page at the front of links
			function persistent_login_add_settings_link( $links ) {
			    $settings_link = '<a href="options-general.php?page=wp-persistent-login">' . __( 'Settings' ) . '</a>';
			    array_unshift($links, $settings_link);
			  	return $links;
			}
			$plugin = plugin_basename( __FILE__ );
			add_filter( "plugin_action_links_$plugin", 'persistent_login_add_settings_link' );
			
	/* ------------------------------------------------------------------------ *
	 * END plugin admin
	 * ------------------------------------------------------------------------ */
	 
		
		
	/* ------------------------------------------------------------------------ *
	 * Options page
	 * ------------------------------------------------------------------------ */
	 
		// load up options.php to create options page
			if( is_admin() ) :
				include(plugin_dir_path(__FILE__) . 'options.php');
			endif;

	/* ------------------------------------------------------------------------ *
	 * END options page
	 * ------------------------------------------------------------------------ */	
	
		
		
	/* ------------------------------------------------------------------------ *
	 * Logic - Check for persistent login
	 * ------------------------------------------------------------------------ */
	 
		/*
			2 methods for auto-login
				
				method 1: when a user visits the website, check if they have logged in before
				method 2: when a user logs in, we set the cookie and add them to the db
		*/
		 
		// method 1
		
			// if the user isn't logged in, check for a valid cookie
		    function persistent_login_check() {
		    
		    // fetch the current roles which are allowed persistent login
				$persistent_login_roles = get_option( 'persistent_login_options_user_access' );
				
				if( $persistent_login_roles ) :
				
				  if( !is_user_logged_in() ) :
							
						// check if user has cookies
						if ( isset($_COOKIE['persistent_login_i']) && isset($_COOKIE['persistent_login_k']) ) :
												
							// store cookie info
							$id = $_COOKIE['persistent_login_i'];
							$key = $_COOKIE['persistent_login_k'];
							
							// store user agent
							$user_agent = sha1($_SERVER['HTTP_USER_AGENT']);
							
							// check if user is in db
							global $wpdb;
							global $tableRef;
							$table = $wpdb->prefix . $tableRef;
							
							$user_check = $wpdb->get_row(
								$wpdb->prepare(
									"SELECT * FROM $table 
									WHERE user = %d 
									AND login_key = %s
									AND user_agent = %s",
									$id, $key, $user_agent
								)
							);
							
							
							// if valid user is in db
							if( !empty($user_check) ) :
													
								// check the login applied to this users role
								$user = get_user_by( 'id', $id );
								
								if( !empty(array_intersect($user->roles, $persistent_login_roles)) ) :
									
									// log user in
										wp_set_current_user( $id, $user->user_login );
										wp_set_auth_cookie( $id );						
										do_action( 'wp_login', $user->user_login );
									
								else :
									
									// user role can no longer be automatically logged-in, delete the cookies
									unset($_COOKIE['persistent_login_i']);
							    setcookie('persistent_login_i', '', time() - 3600);

									unset($_COOKIE['persistent_login_k']);
									setcookie('persistent_login_k', '', time() - 3600);
									
									unset($_COOKIE['persistent_login_d']);
									setcookie('persistent_login_d', '', time() - 3600);
								
								endif;
								
							
							else : 
							
								// double check that they're using the correct combination of key and user ID but the wrong browser
								$user_double_check = $wpdb->get_row(
									$wpdb->prepare(
										"SELECT * FROM $table 
										WHERE user = %d
										AND login_key = %s",
										$id, $key
									)
								);
								
								if( !empty($user_double_check) ) :
							
									/* 
										User and login_key match found, but wrong device. Could be trouble!
									*/
									
									// Delete row from database
										$wpdb->delete( $table, array( 'user' => $id), array( '%d' ) );
										error_log("Attempted security breach detected by WordPress Persistent Login. database row deleted, cookies deleted, user and admin notified.");
									
									// remove cookies for user
										unset($_COOKIE['persistent_login_i']);
								    setcookie('persistent_login_i', '', time() - 3600);
		
										unset($_COOKIE['persistent_login_k']);
										setcookie('persistent_login_k', '', time() - 3600);
										
										unset($_COOKIE['persistent_login_d']);
										setcookie('persistent_login_d', '', time() - 3600);
									
									// e-mail user to notify them
										$user = get_user_by( 'id', $id );
								    $user_email = $user->user_email;
								    
								    $username = $user->user_login;
								    $site_name = get_bloginfo('name');
								    $site_url = get_bloginfo('url');
										
										// send user an email alert
									    $to = $user_email;
									    $subject = "Security Notification from ". $site_name .' ('.$site_url.')';
									    $body = '
					              <h1>Security Notification</h1><br/>
					              <p>Hi '.$username.',</p>
					              <p>This is an automated email to notify you that we\'ve noticed an unusual login attempt on your '. $site_name .' ('. $site_url .') account just now. </p>
					              <h3>I just tried to login</h3>
					              <p>If you have just tried to login yourself, then we apologies for the misunderstanding, you can ignore this email and login with your username and password.</p>
					              <h3>It wasn\'t me, what do I do?</h3>
					              <p>If you haven\'t tried to login yourself, don\'t worry. No one has accessed your account.</p>
					              <p>It is recommended that you re-set your password on '. $site_name .' ('.$site_url.') to ensure the security of your account.</p>
												<br/><br/>
					              <em>
						              <p>Kind Regards,<br/>'. 
						              $site_name .'<br/>
						              <a href="'. $site_url .'">'.$site_url.'</a>
						              </p>
					              </em>
									    ';
									    $headers = array('Content-Type: text/html; charset=UTF-8');
									    
									    if(wp_mail($to, $subject, $body, $headers)) :
									    	error_log('sent warning email to user');
									    else :
									    	error_log('failed to send warning email to user');
									    endif;
									  									
								endif; // end if user double check
							
							endif; // end if user check
						
						else :
						
							// no cookies, nothing to do here
							
						endif; // end if cookies
									
					endif; // end if is logged in
				
				// if there are no persistent login roles, remove any cookies just incase
				else : 
				
					if ( isset($_COOKIE['persistent_login_i']) ) :
						unset($_COOKIE['persistent_login_i']);
		        setcookie('persistent_login_i', '', time() - 3600);
	        endif;
					
					if( isset($_COOKIE['persistent_login_k']) ) :
						unset($_COOKIE['persistent_login_k']);
						setcookie('persistent_login_k', '', time() - 3600);
					endif;
					
					if( isset($_COOKIE['persistent_login_d']) ) :
						unset($_COOKIE['persistent_login_d']);
						setcookie('persistent_login_d', '', time() - 3600);
					endif;
					
				endif; // end if persistent login roles
			}
			add_action('wp', 'persistent_login_check');
	
	/* ------------------------------------------------------------------------ *
	 * END check for persistent login
	 * ------------------------------------------------------------------------ */	
	
	
	
	/* ------------------------------------------------------------------------ *
	 * Setup persistent login
	 * ------------------------------------------------------------------------ */ 
		
		// method 2 - this also runs after method 1
		
			// when a user is logged in, set/reset their cookie
			function persistent_login_set_user_cookie($user_login) {
				
				// fetch the current roles which are allowed persistent login
				$persistent_login_roles = get_option( 'persistent_login_options_user_access' );
					
				if( $persistent_login_roles ) :
					
					// get user info
					$user = get_user_by('login', $user_login);
					$id = $user->ID;
					
					// check this user level can have persistent login
					if( !empty(array_intersect($user->roles, $persistent_login_roles)) ) :
						
						// get user IP
						$ip = $_SERVER["REMOTE_ADDR"];
						$ip = ip2long($ip);
						
						// get user agent
						$user_agent = $_SERVER['HTTP_USER_AGENT'];
						
						// generate new key for user
						$salt = wp_generate_password(20); // 20 character "random" string
						$key = sha1($salt . uniqid(time(), true));
						$device = sha1($user_agent);
									
						// set new cookies
						setcookie("persistent_login_i", $id, time()+31536000);  /* expire in 1 year */
						setcookie("persistent_login_k", $key, time()+31536000);  /* expire in 1 year */
						setcookie("persistent_login_d", $device, time()+31536000);  /* expire in 1 year */

						// check if user is in db
						global $wpdb;
						global $tableRef;
						$table = $wpdb->prefix . $tableRef;
						$user_check = $wpdb->get_results(
							$wpdb->prepare(
								"
								SELECT * FROM $table 
								WHERE user = %d
								AND user_agent = %s
								",
								$id, $device
							) 
						);
					
						// if user is already in db
						if( !empty($user_check) ) :
												
								// update the db
								$update_user_row = $wpdb->update( 
									$table, 
									array('login_key' => $key, 'ip' => $ip), // data - update key and ip
									array('user' => $id, 'user_agent' => $device), // where - for this user on this device
									array('%s', '%d'), // data format
									array('%d', '%s') // where format
								);
								
								if( $update_user_row === false ) :
									error_log('Failed to update user row in database.');
								endif;
						
						else : // user isn't in database
			
							// add new row to db
							$add_new_user_row = $wpdb->insert( 
								$table, 
								array('user' => $id, 'login_key' => $key, 'ip' => $ip, 'user_agent' => $device), // data
								array('%d', '%s', '%d', '%s') // format
							);
							
							if( $add_new_user_row === false ) : 
									error_log('Failed to add new user row to database.');
							endif;  
				
						endif; // end if user check
						
					endif; // end if user level can have persistent login
					
				endif; // end if persistent login roles
								
			}
			add_action('wp_login', 'persistent_login_set_user_cookie', 10, 1);
		
	/* ------------------------------------------------------------------------ *
	 * END setup persistent login
	 * ------------------------------------------------------------------------ */
	
	
	
	/* ------------------------------------------------------------------------ *
	 * Logout - remove persistent login
	 * ------------------------------------------------------------------------ */	
		
		// remove the users cookie and db row when they click logout
			function persistent_login_remove_user_cookie() {
				
				// store variables required
					$user = wp_get_current_user();
					$user_agent = sha1($_SERVER['HTTP_USER_AGENT']);
				
				// remove cookie
					unset($_COOKIE['persistent_login_i']);
	        unset($_COOKIE['persistent_login_k']);
	        unset($_COOKIE['persistent_login_d']);
	        setcookie('persistent_login_i', '', time() - 3600);
	        setcookie('persistent_login_k', '', time() - 3600);
	        setcookie('persistent_login_d', '', time() - 3600);
		        
        // remove row
        	// get global vars
	        global $wpdb;
	        global $tableRef;
	        
	        // delete the row for this user on this device, keeps all other devices logged in
	        $wpdb->delete( 
	        	$wpdb->prefix . $tableRef, 
	        	array('user' => $user->ID, 'user_agent' => $user_agent), // data
	        	array('%d', '%s') // data format
	        );
				
			}
			add_action('clear_auth_cookie', 'persistent_login_remove_user_cookie', 10);
			
	/* ------------------------------------------------------------------------ *
	 * END logout - remove persistent login
	 * ------------------------------------------------------------------------ */
	 
	
endif; // end if persistent login exists

?>